import os
import subprocess
import logging
import logging.handlers
import sqlite3
import time
import sys

class listener:
	def __init__(self):
		self.scriptpath = "/usr/share/lliurex-sync-server"
		self.loginscript = self.scriptpath + "/login/"
		self.logoutscript = self.scriptpath + "/logout/"
		self.accesslogdir = "/var/lib/lliurex-sync-server/"
		self.accesslog = self.accesslogdir+"userlogged"
		self.sysloger = logging.getLogger('sysloger')
		# logging levels = CRITICAL << ERROR << WARNING << INFO << DEBUG << NOTSET
		self.sysloger.setLevel(logging.DEBUG)
		self.sysloger.addHandler(logging.handlers.SysLogHandler(address='/dev/log'))
		self.blocksqlite = False
		if not os.path.exists(self.accesslog) :
			self.sysloger.warning("[lliurex-listener] : Creating database for users log")
			retcode = subprocess.call(["touch",self.accesslog])
			if retcode == 1:
				self.sysloger.error("[lliurex-listener] : Can't create database for users log on "+self.accesslogdir)
				sys.exit(1)
			conn = sqlite3.connect(self.accesslog)
			c = conn.cursor()
			c.execute('create table users (machine text, user text, lasttime real,  logtime real, mac text)')
			conn.commit()
			c.close()
			conn.close()
			self.sysloger.info("[lliurex-listener] : Database for user loggin created")
		#
		#
		#	Check is table users exist
		#
		#
		paux = subprocess.Popen(["llxcfg-showvars","CAP_ZERO_CAPNAME"],stdout = subprocess.PIPE,stderr = subprocess.PIPE)
		cnaux = paux.stdout.readline()
		if len(cnaux) == 0:
			self.sysloger.error("[lliurex-listener] : CAP_ZERO_CAPNAME is not defined ")
			sys.exit(1)
		else:

			self.capname = cnaux.split('=')[1]
			self.capname = str(self.capname[1:-3])
		self.sysloger.debug("[lliurex-listener] : CAPNAME is " + self.capname)


	def userlogin(self,username,ip,mac='00:00:00:00:00:00'):
		#syslog write
		
		self.sysloger.info("[lliurex-listener] : User " + username + " from " + ip +" loggin")
		logtime = time.time()
		capname = self.capname
		#Register user on database
		while self.blocksqlite :
			time.sleep(0.2)
		self.blocksqlite = True
		conn = sqlite3.connect(self.accesslog)
		c = conn.cursor()
		t = (ip,username,logtime,logtime,mac)
		self.sysloger.debug("[lliurex-listener] : insert into users user:" + username + ",machine:" + ip + ",logtime:" + str(logtime) + ",mac:" + mac)
		c.execute("insert into users values (?,?,?,?,?)",t)
		conn.commit()
		c.close()
		conn.close()
		self.blocksqlite = False
		#Execute other scripts
		if os.path.exists(self.loginscript) :
			for script in os.listdir(self.loginscript) :
				self.sysloger.info('[lliurex-listener] : Executing ' + script +' ... ')
				try:
					execfile(self.loginscript+script)
					self.sysloger.info('[lliurex-listener] : Executed ' + script +' ... ')
				except:
					self.sysloger.error('[lliurex-listener] : Script ' + script +' not executed on login ')
		else: 
			print ("no hay scripts para ejecutar")
		return 'ok'
		
	def userlogout(self,username,ip,mac='00:00:00:00:00:00'):
		#syslog write
		self.sysloger.info("[lliurex-listener] : User " + username + " from " + ip +" logout")
		capname = self.capname
		#Delete user on database
		while self.blocksqlite :
			time.sleep(0.2)
		self.blocksqlite = True
		conn = sqlite3.connect(self.accesslog)
		c = conn.cursor()
		t = (ip,username)
		self.sysloger.debug("[lliurex-listener] : delete from users user:"+username+" on machine:"+ip)
		c.execute("delete from users where machine =? and user =?",t)
		conn.commit()
		c.close()
		conn.close()
		self.blocksqlite = False
		#Execute other scripts
		if os.path.exists(self.logoutscript) :
			for script in os.listdir(self.logoutscript) :
				self.sysloger.info('[lliurex-listener] : Executing '  + script +' ... ')
				try :
					execfile(self.logoutscript+script)
					self.sysloger.info('[lliurex-listener] : Executed ' + script +' ... ')
				except:
					self.sysloger.error('[lliurex-listener] : Script ' + script +' not executed on logout')
		
		
		return 'ok'

	def machineSignLife(self,ip,mac='00:00:00:00:00:00'):
		#syslog write
		self.sysloger.info("[lliurex-listener] : Machine from "+ ip +" is alive")
		logtime = time.time()
		capname = self.capname
		#Update times on database
		while self.blocksqlite :
			time.sleep(0.2)
		self.blocksqlite = True
		conn = sqlite3.connect(self.accesslog)
		c = conn.cursor()
		t = (logtime,ip)
		self.sysloger.debug("[lliurex-listener] : update lasttime for users from machine:" + ip)
		c.execute("update users set lasttime =? where machine =?",t)
		conn.commit()
		c.close()
		conn.close()
		self.blocksqlite = False
		return 'ok'
